///////////////////////////////////////////////////////////////////////////////
//
// libRTEA
//   : a encrypt/decrypt class tool for RAW data.
// ===========================================================================
//
// (C) Copyright 2011 ,  rageworx@gamil.com
//
///////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libRTEA.h"

#define  SUM_INIT       0xC6EF3720
#define  DELTA          0x9E3779B9
#define  PAD_BYTE       0xA5

void RTEAcodec::encipher(const unsigned long *v,unsigned long *w, const unsigned long *k)
{
    register unsigned long  y     = v[0];
    register unsigned long  z     = v[1];
    register unsigned long  sum   = 0;
    register unsigned long  delta = DELTA;
    register unsigned long  n     = 32;

    while(n-->0)
    {
        y += (z << 4 ^ z >> 5) + z ^ sum + k[sum&3];
        sum += delta;
        z += (y << 4 ^ y >> 5) + y ^ sum + k[sum>>11 & 3];
    }

    w[0]=y; w[1]=z;
}

void RTEAcodec::decipher(const unsigned long *v,unsigned long *w, const unsigned long *k)
{
    register unsigned long y     = v[0];
    register unsigned long z     = v[1];
    register unsigned long sum   = SUM_INIT;
    register unsigned long delta = DELTA;
    register unsigned long n     = 32;

   /* sum = delta<<5, in general sum = delta * n */

    while(n-->0)
    {
        z -= (y << 4 ^ y >> 5) + y ^ sum + k[sum>>11 & 3];
        sum -= delta;
        y -= (z << 4 ^ z >> 5) + z ^ sum + k[sum&3];
    }

    w[0]=y; w[1]=z;
}

void RTEAcodec::EncodeBuffer(char* BuffSrc, char* BuffDst, int BufferLen, teaKeyString* Key)
{
    int nNewBufLen = ( BufferLen + 1 ) / 2;
    int buffQue    = 0;

    // check key padding.
    for(int cnt=0; cnt<16; cnt++)
    {
        if(*Key[cnt] == 0)
            *Key[cnt] = PAD_BYTE;
    }

    while(nNewBufLen > 0)
    {
        encipher((const unsigned long*)&BuffSrc[buffQue],
                 (unsigned long*)&BuffDst[buffQue],
                 (const unsigned long*)Key);
        buffQue += 8;
        nNewBufLen--;
    }
}

void RTEAcodec::Decodebuffer(char* BuffSrc, char* BuffDst, int BufferLen, teaKeyString* Key)
{
    int nNewBufLen = ( BufferLen + 1 ) / 2;
    int buffQue    = 0;

    // check key padding.
    for(int cnt=0; cnt<16; cnt++)
    {
        if(*Key[cnt] == 0)
            *Key[cnt] = PAD_BYTE;
    }

    while(nNewBufLen > 0)
    {
        decipher((const unsigned long*)&BuffSrc[buffQue],
                 (unsigned long*)&BuffDst[buffQue],
                 (const unsigned long*)Key);
        buffQue += 8;
        nNewBufLen--;
    }
}
